{{
***************************************
* File jvmLogic: logical instructions *
***************************************
}}

{{
  Logical instructions include operations to shift values and perform bitwise and, bitwise or, and bitwise xor. 
  To perform logical functions, both values must be of the same type.  

  Shifting Values. 
  When values are left-shifted, the sign of the value is not affected. However, when values are right-shifted, 
  the sign of the value can be affected. For this reason, values are right-shifted in one of two ways: with the 
  sign extension (called arithmetic shifting) or without the sign extension (called logical or unsigned shifting).
    
  For right arithmetic shifting, the first value is generally shifted by the amount indicated by the low 6 bits 
  of the second value. For right logical shifting, the first value is generally shifted by the amount indicated 
  by the low 5 bits of the second value.

  For bitwise operations, two values are popped off the stack and replaced on the stack by their bitwise result.  
}}


OBJ
  js: "jvmStack"
  jd: "jvmData"
  
  
PUB jb_ishl | w1,w2
{{ Meaning: Integer shift left. }}
{{
  JEM format: ishl
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  w2 <<= w1
  js.jb_push(w2)
  jd.incJvmPC(1)
  

PUB jb_ishr | w1,w2
{{ Meaning: Integer arithmetic shift right. }}
{{
  JEM foramt: ishr
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  w2 ~>= w1
  js.jb_push(w2)
  jd.incJvmPC(1)
  

PUB jb_iushr | w1,w2
{{ Meaning: Integer logical shift right. }}
{{
  JEM format: iushr
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  w2 >>= w1
  js.jb_push(w2)
  jd.incJvmPC(1)
  

{{PUB jb_lshl}}
{{ Meaning: Long integer shift left. }}

{{PUB jb_lshr}}
{{ Meaning: Long integer arithmetic shift right. }}

{{PUB jb_lushr}}
{{ Meaning: Long integer logical shift right. }}


PUB jb_iand | w1,w2
{{ Meaning: Integer bitwise and. }}
{{
  JEM format: iand
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  w2 &= w1
  js.jb_push(w2)
  jd.incJvmPC(1)
  

{{PUB jb_land}}
{{ Meaning: Long integer bitwise and. }}


PUB jb_ior | w1,w2
{{ Meaning: Integer bitwise or. }}
{{
  JEMformat: ior
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  w2 |= w1
  js.jb_push(w2)
  jd.incJvmPC(1)
  

{{PUB jb_lor}}
{{ Meaning: Long integer bitwise or. }}


PUB jb_ixor | w1,w2
{{ Meaning: Integer bitwise xor. }}
{{
  JEM format: ixor
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  w2 ^= w1
  js.jb_push(w2)
  jd.incJvmPC(1)
  

{{PUB jb_lxor}}
{{ Meaning: Long integer bitwise xor. }}

